﻿using System.IdentityModel.Selectors;
using System.IdentityModel.Tokens;
using System.Text;
using System.Xml;
using System.Security.Cryptography;

namespace WCF_ExactTarget_WebSample.Helper
{
    internal class CustomTokenSerializer : SecurityTokenSerializer
    {
        #region Override

        /// <summary>
        /// Initializes a new instance of the <see cref="CustomTokenSerializer" /> class.
        /// </summary>
        public CustomTokenSerializer() 
        {
        }

        /// <summary>
        /// Writes the token core.
        /// </summary>
        /// <param name="writer">The writer.</param>
        /// <param name="token">The token.</param>
        protected override void WriteTokenCore(XmlWriter writer, SecurityToken token)
        {
            string tokennamespace = "o";

            DateTime created = DateTime.UtcNow;
            DateTime createdTimestamp = DateTime.UtcNow;
            DateTime expiresTimestanp = createdTimestamp.AddSeconds(300);

            string createdStr = created.ToString("yyyy-MM-ddTHH:mm:ss.fffZ");
            string createdTimestampStr = createdTimestamp.ToString("yyyy-MM-ddTHH:mm:ss.fffZ");
            string expiresTimestanpStr = expiresTimestanp.ToString("yyyy-MM-ddTHH:mm:ss.fffZ");

            string phrase = Guid.NewGuid().ToString();
            var nonce = GetSHA1String(phrase);

            string password = CreatePasswordDigest(Convert.FromBase64String(nonce), createdStr, CustomCredentials.PASSWORD);

            writer.WriteRaw(string.Format(
                "<u:Timestamp>" +
                "<u:Created>" + createdTimestampStr + "</u:Created>" +
                "<u:Expires>" + expiresTimestanpStr + "</u:Expires>" +
                "</u:Timestamp>" +
            "<{0}:UsernameToken u:Id=\"" + token.Id + "\" xmlns:u=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd\">" +
            "<{0}:Username>" + CustomCredentials.USERNAME + "</{0}:Username>" +
            "<{0}:Password Type=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordDigest\">" +
            password + "</{0}:Password>" +
            "<{0}:Nonce EncodingType=\"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary\">" +
            nonce + "</{0}:Nonce>" +
            "<u:Created>" + createdStr + "</u:Created></{0}:UsernameToken>", tokennamespace));
        }

        /// <summary>
        /// Gets the sh a1 string.
        /// </summary>
        /// <param name="phrase">The phrase.</param>
        /// <returns></returns>
        protected static string GetSHA1String(string phrase)
        {
            byte[] hashedDataBytes = SHA1.HashData(Encoding.UTF8.GetBytes(phrase));
            return Convert.ToBase64String(hashedDataBytes);
        }

        /// <summary>
        /// Creates the password digest.
        /// </summary>
        /// <param name="nonce">The nonce.</param>
        /// <param name="createdTime">The created time.</param>
        /// <param name="password">The password.</param>
        /// <returns></returns>
        protected static string CreatePasswordDigest(byte[] nonce, string createdTime, string password)
        {
            // combine three byte arrays into one
            byte[] time = Encoding.UTF8.GetBytes(createdTime);
            byte[] pwd = Encoding.UTF8.GetBytes(password);
            byte[] operand = new byte[nonce.Length + time.Length + pwd.Length];
            Array.Copy(nonce, operand, nonce.Length);
            Array.Copy(time, 0, operand, nonce.Length, time.Length);
            Array.Copy(pwd, 0, operand, nonce.Length + time.Length, pwd.Length);

            // create the hash
            byte[] hashedDataBytes = SHA1.HashData(operand);
            return Convert.ToBase64String(hashedDataBytes);
        }

        #endregion

        #region Inherited

        protected override bool CanReadKeyIdentifierClauseCore(XmlReader reader)
        {
            throw new NotImplementedException();
        }

        protected override bool CanReadKeyIdentifierCore(XmlReader reader)
        {
            throw new NotImplementedException();
        }

        protected override bool CanReadTokenCore(XmlReader reader)
        {
            throw new NotImplementedException();
        }

        protected override bool CanWriteKeyIdentifierClauseCore(SecurityKeyIdentifierClause keyIdentifierClause)
        {
            throw new NotImplementedException();
        }

        protected override bool CanWriteKeyIdentifierCore(SecurityKeyIdentifier keyIdentifier)
        {
            throw new NotImplementedException();
        }

        protected override bool CanWriteTokenCore(SecurityToken token)
        {
            throw new NotImplementedException();
        }

        protected override SecurityKeyIdentifierClause ReadKeyIdentifierClauseCore(XmlReader reader)
        {
            throw new NotImplementedException();
        }

        protected override SecurityKeyIdentifier ReadKeyIdentifierCore(XmlReader reader)
        {
            throw new NotImplementedException();
        }

        protected override SecurityToken ReadTokenCore(XmlReader reader, SecurityTokenResolver tokenResolver)
        {
            throw new NotImplementedException();
        }

        protected override void WriteKeyIdentifierClauseCore(XmlWriter writer, SecurityKeyIdentifierClause keyIdentifierClause)
        {
            throw new NotImplementedException();
        }

        protected override void WriteKeyIdentifierCore(XmlWriter writer, SecurityKeyIdentifier keyIdentifier)
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}
