﻿using KBOServiceReference;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.ServiceModel;

internal class Program
{
    private const string _serviceUrl = "https://kbopub-acc.economie.fgov.be/kbopubws110000/services/wsKBOPub";

    public static void Main()
    {        
        try
        {
            Console.WriteLine("Welcome to KBO Consult !");

            // ****************************************** Read Profile ******************************************

            Console.WriteLine("Type your user name : ");
            string? username = string.Empty;
            username = Console.ReadLine();

            Console.WriteLine("Type your password: ");
            string? password = string.Empty;
            password = Console.ReadLine();

            Console.WriteLine("Type the enterprise number: ");
            string? enterpriseNumber = string.Empty;
            enterpriseNumber = Console.ReadLine();

            // ****************************************** END Read Profile ***************************************



            // ****************************************** Create a request ******************************************

            // Create a client
            WSKBOPubChannel client = CreateRealTimeOnlineProxy(username, password);
            
            // Create a request
            ReadEnterpriseRequestType requestType = new();
            RequestContextType requestContext = new()
            {
                Id = "ABCD",
                Language = new RequestContextTypeLanguage[] { RequestContextTypeLanguage.nl }
            };
            requestType.EnterpriseNumber = Convert.ToInt64(enterpriseNumber);
            ReadEnterpriseRequest request = new(requestContext, requestType);

            // Execute the request
            ReadEnterpriseResponse response = client.ReadEnterprise(request);

            Console.WriteLine($"Success - Enterprise name confirmation : {response.ReadEnterpriseReply.Enterprise.Number}");

            // ****************************************** Request created ***************************************
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex);
        }
        Console.ReadLine();
    }

    /// <summary>
    /// Creates the real time online proxy.
    /// </summary>
    /// <param name="username">The username.</param>
    /// <param name="password">The password.</param>
    /// <returns></returns>
    private static WSKBOPubChannel CreateRealTimeOnlineProxy(string username, string password)
    {
        CustomBinding customBinding = new();
        var security = TransportSecurityBindingElement.CreateUserNameOverTransportBindingElement();
        security.IncludeTimestamp = false;
        security.DefaultAlgorithmSuite = SecurityAlgorithmSuite.Basic256;
        security.MessageSecurityVersion = MessageSecurityVersion.WSSecurity10WSTrustFebruary2005WSSecureConversationFebruary2005WSSecurityPolicy11BasicSecurityProfile10;
        TextMessageEncodingBindingElement encoding = new()
        {
            MessageVersion = MessageVersion.Soap11
        };

        var transport = new HttpsTransportBindingElement
        {
            MaxReceivedMessageSize = 20000000 // 20 megs
        };

        System.Net.ServicePointManager.SecurityProtocol = System.Net.SecurityProtocolType.Tls12;
        customBinding.Elements.Add(security);
        customBinding.Elements.Add(encoding);
        customBinding.Elements.Add(transport);

        // Create the client
        ChannelFactory<WSKBOPubChannel> factory = new(customBinding, new EndpointAddress(new Uri(_serviceUrl)));

        // Clean Default EndPointBehaviour
        if(factory.Endpoint.EndpointBehaviors.Count > 0)
        {
            //Remobe default client credential
            factory.Endpoint.EndpointBehaviors.Remove(factory.Endpoint.EndpointBehaviors[0]);
        }

        // Add new EndPointBehaviour with custom credential for KBO
        factory.Endpoint.EndpointBehaviors.Add(new CustomCredentials(username, password));

        // Add user credential  into the client
        factory.Credentials.UserName.UserName = username;
        factory.Credentials.UserName.Password = password;

        // Create the channel
        WSKBOPubChannel channel = factory.CreateChannel();
        return channel;
    }
}